@@ -1,5 +1,5 @@
 
-#   $Id: InterBase.pm,v 1.19 2001/03/25 01:44:11 edpratomo Exp $
+#   $Id: InterBase.pm,v 1.26 2001/08/01 01:03:32 danielritz Exp $
 #
 #   Copyright (c) 1999-2001 Edwin Pratomo
 #
@@ -20,7 +20,7 @@ require Exporter;
 require DynaLoader;
 
 @ISA = qw(Exporter DynaLoader);
-$VERSION = '0.25.1';
+$VERSION = '0.28.4';
 
 bootstrap DBD::InterBase $VERSION;
 
@@ -357,15 +357,13 @@ controls how InterBase interprets:
  - new 6.0 reserved keywords
 
 Valid values for B<ib_dialect> are 1, 2, and 3. The driver's default value is
-1. For more detail reading on this topic, please refer to:
-
- http://www.interbase.com/open/research/art_60dialects.html
+1. 
 
 B<ib_role> specifies the role of the connecting user. B<SQL role> is
 implemented by InterBase to make database administration easier when dealing
 with lots of users. A detailed reading can be found at:
 
- http://www.interbase.com/downloads/sqlroles.pdf
+ http://www.ibphoenix.com/ibp_sqlroles.html
 
 If B<ib_cache> is not specified, the default database's cache size value will be 
 used. The InterBase Operations Guide discusses in full length the importance of 
@@ -798,9 +796,7 @@ start a new transaction. A disconnect will issue a rollback.
 InterBase provides fine control over transaction behavior, where users can
 specify the access mode, the isolation level, the lock resolution, and the 
 table reservation (for a specified table). For this purpose,
-C<set_tx_param()> database handle method is available. Please notice that
-this private method is new and considered B<experimental> at this release.
-Use with caution!
+C<set_tx_param()> database handle method is available. 
 
 Upon a successful C<connect()>, these default parameter values will be used
 for every SQL operation:
@@ -816,10 +812,10 @@ Any of the above value can be changed using C<set_tx_param()>.
 =item B<set_tx_param> 
 
  $dbh->func( 
-	-access_mode     => 'read_write',
-	-isolation_level => 'read_committed',
-	-lock_resolution => 'wait',
-	'set_tx_param'
+    -access_mode     => 'read_write',
+    -isolation_level => 'read_committed',
+    -lock_resolution => 'wait',
+    'set_tx_param'
  );
 
 Valid value for C<-access_mode> is C<read_write>, or C<read_only>. 
@@ -830,7 +826,7 @@ C<record_version> or C<no_record_version>, then they should be inside an
 anonymous array:
 
  $dbh->func( 
-	-isolation_level => ['read_committed', 'record_version']
+    -isolation_level => ['read_committed', 'record_version'],
     'set_tx_param'
  );
 
@@ -876,10 +872,7 @@ fetch() methods.
 =head2 Compatibility with DBI Extension modules 
 
 C<DBD::InterBase> is known to work with C<DBIx::Recordset> 0.21, and
-C<Apache::DBI> 0.87. DBI 1.14 has a subtle bug on C<fetchall_arrayref> method,
-when it is passed an empty hash ref. The patch is included here as 
-DBI.pm.diff, because it is required to make the driver work with 
-C<DBIx::Tree> 0.91. Yuri Vasiliev <I<yuri.vasiliev@targuscom.com>> reported 
+C<Apache::DBI> 0.87. Yuri Vasiliev <I<yuri.vasiliev@targuscom.com>> reported 
 successful usage with Apache::AuthDBI (part of C<Apache::DBI> 0.87 
 distribution).
 
@@ -888,10 +881,31 @@ C<Tie::DBI>. C<Tie::DBI> calls $dbh->prepare("LISTFIELDS $table_name") on
 which InterBase fails to parse. I think that the call should be made within 
 an eval block.
 
-=head2 Tested Platforms
+=head1 TESTED PLATFORMS
 
-Client: Linux, glibc-2.1.2, x86 egcs-1.1.2, kernel 2.2.12-20. 
-Server: InterBase 6.0 SS and Classic for Linux, InterBase 6.0 for Windows.
+=head2 Client
+
+=over 4
+
+=item Linux, glibc-2.1.2, x86 egcs-1.1.2, kernel 2.2.12-20. 
+
+=item FreeBSD
+
+=item SPARC Solaris
+
+=item Win32
+
+=back
+
+=head2 Server
+
+=over 4
+
+=item InterBase 6.0 SS and Classic for Linux
+
+=item InterBase 6.0 for Windows, FreeBSD, SPARC Solaris
+
+=back
 
 =head1 AUTHOR
 
@@ -902,9 +916,12 @@ Server: InterBase 6.0 SS and Classic for Linux, InterBase 6.0 for Windows.
 Partially based on the work of Bill Karwin's IBPerl, Jochen Wiedmann's
 DBD::mysql, and Edmund Mergl's DBD::Pg.
 
+Daniel Ritz <daniel_ritz@gmx.ch> works on SPARC Solaris and Win32 ports.
+
 =head1 BUGS
 
-Known problem with BLOB fields on Win32 platform.
+Uses of nested statement handles break under AutoCommit mode. As the
+workaround, try to turn off AutoCommit attribute inside a block. 
 
 =head1 SEE ALSO
 
@@ -926,7 +943,6 @@ development days of this XS version.
 
 Michael Arnett <I<marnett@mediaone.net>>, Flemming Frandsen <I<dion@swamp.dk>>,
 Mike Shoyher <I<msh@e-labs.ru>>, Christiaan Lademann <I<cal@zls.de>> sent me 
-patches for the first version. Their code are still used in the current
-version, with or without some modification.
+patches for the first version. 
 
 =cut
